/*
 * reg.c
 *
 * Copyright (C) 1998, 1999 Rasca, Berlin
 * EMail: thron@gmx.de
 *
 * Olivier Fourdan (fourdan@xfce.org)
 *
 * Edscott Wilson Garcia 2001-2003 for xfce project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#include <errno.h>
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_TIME_H
#include <time.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <glib.h>

#ifdef XFCE_SUPPORT
#include <libxfce4util/util.h>
#endif

#include "reg.h"

static char regfile[_POSIX_PATH_MAX];
static GList *reg_list = NULL;

/*
 */
static int compare_sfx(char *sfx1, char *sfx2)
{
    int r;
    char *asfx1 = g_strdup(sfx1);
    char *asfx2 = g_strdup(sfx2);
    if(strchr(asfx1, '-'))
	asfx1 = strtok(asfx1, "-");
    if(strchr(asfx2, '-'))
	asfx2 = strtok(asfx2, "-");
    r = strcmp(asfx1, asfx2);
    g_free(asfx1);
    g_free(asfx2);
    return r;
}

static int compare_reg(reg * reg1, reg * reg2)
{
    return compare_sfx(reg1->sfx, reg2->sfx);
}

void reg_free_reg(reg_t * app)
{
    if(!app)
	return;
    if(app->app)
	g_free(app->app);
    if(app->arg)
	g_free(app->arg);
    if(app->sfx)
	g_free(app->sfx);
    g_free(app);
}

/*
 * on startup we build a complete list
 */
void reg_build_list(void)
{
#define BUFSIZE 1024
    FILE *fp;
    char buf[BUFSIZE], *p, *ep, *buf_end;
    char suffix[_POSIX_PATH_MAX];
    char cmd[PATH_MAX + 1];
    char arg[PATH_MAX + 1];
    reg_t *prg;
    int inner;
    int line;

    xfce_get_userfile_r(regfile, _POSIX_PATH_MAX-1, "xffm%cxffm.reg",
		    G_DIR_SEPARATOR);
    fp = fopen(regfile, "r");
    if(!fp)
    {
	printf("Warning, can't open %s\n", regfile);
	return;
    }
    buf_end = buf + BUFSIZE - 1;
    line = 0;
    while(fgets(buf, BUFSIZE, fp) != NULL)
    {
	line++;
	p = buf;
	while((*p == ' ') || (*p == '\t'))
	    p++;
	if(*p == '#')
	    continue;
	prg = g_malloc(sizeof(reg_t));
	if(!prg)
	    g_assert_not_reached();
	prg->app = prg->sfx = prg->arg = NULL;
	if(*p != '(')
	{
	    /* old style */
	    *arg = '\0';
	    sscanf(buf, "%s %[^ \n] %[^\n]", suffix, cmd, arg);
	    prg->sfx = g_strdup(suffix);
	    prg->app = g_strdup(cmd);
	    if(*arg)
		prg->arg = g_strdup(arg);
	    prg->len = strlen(suffix);
	    reg_list = g_list_append(reg_list, prg);
	}
	else
	{
	    /* new style:
	     * (suffix)(program)(arguments)
	     */
	    inner = 0;
	    p++;
	    ep = p;
	    while(*ep && (ep < buf_end))
	    {
		if(*ep == '(')
		    inner++;
		else if(*ep == ')')
		{
		    if(inner)
			inner--;
		    else
			break;
		}
		ep++;
	    }
	    if(*ep != ')')
	    {
		/* parse error */
		fprintf(stderr, "parse error at line %d\n", line);
		reg_free_reg(prg);
		continue;
	    }
	    *ep = '\0';
	    prg->sfx = g_strdup(p);
	    prg->len = strlen(prg->sfx);

	    /* find program */
	    p = ep + 1;
	    while(*p && (p < buf_end))
	    {
		if(*p == '(')
		    break;
		p++;
	    }
	    if(*p != '(')
	    {
		/* parse error */
		fprintf(stderr, "parse error at line %d\n", line);
		reg_free_reg(prg);
		continue;
	    }
	    inner = 0;
	    p++;
	    ep = p;
	    while(*ep && (ep < buf_end))
	    {
		if(*ep == '(')
		    inner++;
		else if(*ep == ')')
		{
		    if(inner)
			inner--;
		    else
			break;
		}
		ep++;
	    }
	    if(*ep != ')')
	    {
		/* parse error */
		fprintf(stderr, "parse error at line %d\n", line);
		reg_free_reg(prg);
		continue;
	    }
	    *ep = '\0';
	    prg->app = g_strdup(p);

	    /* find args */
	    p = ep + 1;
	    while(*p && (p < buf_end))
	    {
		if(*p == '(')
		    break;
		p++;
	    }
	    if(*p == '(')
	    {
		inner = 0;
		p++;
		ep = p;
		while(*ep && (ep < buf_end))
		{
		    if(*ep == '(')
			inner++;
		    else if(*ep == ')')
		    {
			if(inner)
			    inner--;
			else
			    break;
		    }
		    ep++;
		}
		if(*ep != ')')
		{
		    /* parse error */
		    fprintf(stderr, "parse error at line %d\n", line);
		    reg_free_reg(prg);
		    continue;
		}
		*ep = '\0';
		if(p != ep)
		    prg->arg = g_strdup(p);
	    }
	    /* check for coherent values */
	    /* only if  DISPLAY is :0.0 to avoid false removes from remotes */
	    /* if it is a directory register, does the directory exist? */
	    if(strcmp(getenv("DISPLAY"), ":0.0") == 0)
	    {
		gchar *test;
		if(*(prg->sfx) == '/')
		{
		    struct stat s;
		    if(stat(prg->sfx, &s) < 0)
		    {
			fprintf(stderr, "xftree:removing %s from xtree.reg\n", prg->sfx);
			reg_free_reg(prg);
			continue;
		    }
		}
		/* does the application exist? glob the path */
		test=g_find_program_in_path(prg->app);
		if(!test)
		{
		    fprintf(stderr, "xffm:removing %s application from xtree.reg\n", prg->app);
		    reg_free_reg(prg);
		    continue;
		} else g_free(test);
	    }
	    reg_list = g_list_append(reg_list, prg);
	}
    }
    fclose(fp);
    return;
}

/*
 */
char *reg_app_by_suffix(char *sfx)
{
    GList *tmp_list = reg_list;
    reg *prg;
    while(tmp_list)
    {
	prg = tmp_list->data;
	if(compare_sfx(prg->sfx, sfx) == 0)
	{
	    return (prg->app);
	}
	tmp_list = tmp_list->next;
    }
    return (NULL);
}

/*
 */
reg_t *reg_prog_by_suffix(char *sfx)
{
    reg_t *prg;
    GList *tmp_list = reg_list;
    while(tmp_list)
    {
	prg = tmp_list->data;
	/*printf("DBG: ojo %s %s\n",prg->app,prg->arg); */
	if(compare_sfx(prg->sfx, sfx) == 0)
	{
	    return (prg);
	}
	tmp_list = tmp_list->next;
    }
    return (NULL);
}
reg_t *
reg_prog_by_file (char *file)
{
  reg_t *prg;
  int len;
  GList *tmp_list;

  if (!file)
    return NULL;
  len = strlen (file);

  for(tmp_list = reg_list; tmp_list; tmp_list = tmp_list->next)
  {
    prg = tmp_list->data;
    if ((!prg) || (prg->len > len)) continue;
    if (strcmp (file + (len - prg->len), prg->sfx) == 0) return (prg);
  }
  return (NULL);
}


/*
 */
char *reg_app_by_file(char *file)
{
    reg *prg;
    char *loc;
    GList *tmp_list;
    static char *app_arg = NULL;

    if(app_arg)
	g_free(app_arg);

    if(!file)
	return NULL;

    for(tmp_list = reg_list; tmp_list; tmp_list = tmp_list->next)
    {
	prg = tmp_list->data;
	if(!prg)
	    continue;

	if(strchr(file, '.'))
	    loc = strrchr(file, '.');
	else
	    loc = file;

	if(compare_sfx(loc, prg->sfx) == 0)
	{
	    int len;
	    len = (prg->arg) ? strlen(prg->arg) : 0;
	    len += ((prg->app) ? strlen(prg->app) : 0);
	    len += 2;
	    app_arg = (char *)malloc(len);
	    if(!app_arg)
		g_assert_not_reached();

	    if(prg->arg)
		sprintf(app_arg, "%s %s", prg->app, prg->arg);
	    else
		sprintf(app_arg, "%s", prg->app);

	    return (app_arg);
	}
    }
    return (NULL);
}

void reg_add_suffix(char *sfx, char *program, char *args)
{
    reg *prg = NULL;
    GList *g_tmp = reg_list;
    int found = 0;
    /* is suffix already registered?
     * check it out ..
     */
    while(g_tmp)
    {
	prg = g_tmp->data;
	if(compare_sfx(prg->sfx, sfx) == 0)
	{
	    found = 1;
	    break;
	}
	g_tmp = g_tmp->next;
    }
    if(found)
    {
	g_free(prg->app);
	g_free(prg->arg);
	prg->app = g_strdup(program);
	prg->arg = NULL;
	if(args)
	{
	    prg->arg = g_strdup(args);
	}
    }
    else
    {
	prg = g_malloc(sizeof(reg));
	prg->app = g_strdup(program);
	prg->sfx = g_strdup(sfx);
	prg->len = strlen(sfx);
	if(args)
	{
	    prg->arg = g_strdup(args);
	}
	else
	{
	    prg->arg = NULL;
	}
	reg_list = g_list_append(reg_list, prg);
    }
    if(reg_list)
	reg_list = g_list_sort(reg_list, (GCompareFunc) compare_reg);
    return;
}

/*
 */
void reg_destroy_list(GList * list)
{
    GList *g_tmp = list;
    reg *prg;
    while(g_tmp)
    {
	prg = (reg *) g_tmp->data;
	g_free(prg->sfx);
	g_free(prg->app);
	if(prg->arg)
	    g_free(prg->arg);
	g_free(prg);
	g_tmp = g_tmp->next;
    }
    g_list_free(list);
    list = NULL;
}


/*
 * save registry to named file
 */
int reg_save(void)
{
    FILE *fp;
    reg *prg;
    GList *tmp_list = reg_list;

    if(!regfile)
	return (0);
    fp = fopen(regfile, "w");
    if(!fp)
	return (0);
    fprintf(fp, "# (suffix) (program) (args)\n");
    while(tmp_list)
    {
	prg = tmp_list->data;
	if(prg)
	{
	    /* printf("DBG: writing out %s %s\n",prg->sfx,prg->app); */
	    if(prg->arg)
	    {
		fprintf(fp, "(%s) (%s) (%s)\n", prg->sfx, prg->app, prg->arg);
	    }
	    else
	    {
		fprintf(fp, "(%s) (%s) ()\n", prg->sfx, prg->app);
	    }
	}
	tmp_list = tmp_list->next;
    }
    fclose(fp);
    return (TRUE);
}

/*
 * return a list of all registered applications with arguments
 */
GList *reg_app_list(void)
{
    GList *g_apps = NULL;
    GList *g_tmp;
    char *app_arg;
    GList *tmp_list = reg_list;

    while(tmp_list)
    {
	char *app, *arg;
	int len;
	app = ((reg *) tmp_list->data)->app;
	arg = ((reg *) tmp_list->data)->arg;
	len = (arg) ? strlen(arg) : 0;
	len += strlen(app);
	len += 2;
	app_arg = (char *)malloc(len);
	if(app_arg)
	{
	    if(arg)
		sprintf(app_arg, "%s %s", app, arg);
	    else
		sprintf(app_arg, "%s", app);
	    g_apps = g_list_append(g_apps, app_arg);
	}
	tmp_list = tmp_list->next;
    }
    if(g_apps)
	g_apps = g_list_sort(g_apps, (GCompareFunc) strcmp);
    /* remove dupes */
    g_tmp = g_apps;
    while(g_tmp)
    {
	if(g_tmp->next)
	{
	    if(strcmp((char *)g_tmp->data, (char *)g_tmp->next->data) == 0)
	    {
		char *rem;
		rem = g_tmp->data;
		g_tmp = g_apps = g_list_remove(g_apps, rem);
		g_free(rem);
		continue;
	    }
	}
	if(!g_tmp)
	    break;
	else
	    g_tmp = g_tmp->next;
    }
    return g_apps;
}

GList *reg_app_list_free(GList * g_apps)
{
    char *rem;
    GList *g_tmp;
    g_tmp = g_apps;
    while(g_tmp)
    {
	rem = g_tmp->data;
	if(rem)
	{
	    g_tmp = g_apps = g_list_remove(g_apps, rem);
	    g_free(rem);
	}
	if(!g_tmp)
	    break;
	else
	    g_tmp = g_tmp->next;
    }
    return NULL;
}
